﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using UnityEngine;

#if UNITY_WSA && !UNITY_EDITOR
using Newtonsoft;
#endif

using METDataset;

[CreateAssetMenu(menuName = "EMPlus Dataset/MET")]
public class MET : ScriptableObject
{
    [HideInInspector] [NonSerialized] public bool dataLoaded = false;

    [HideInInspector] [NonSerialized] public DXTObjectList dxtObjects;
    [HideInInspector] [NonSerialized] public TagList tags;

    public string jsonPath = "/Users/voelzow/MET";
    public bool loadDxtObjects = true;
    public bool loadTags = true;

    private T LoadJson<T>(string filename)
    {
        if (!File.Exists(filename))
        {
            Debug.LogError("file " + filename + " does not exist! (path: " + jsonPath + ")");
            return default(T);
        }

#if (!UNITY_WSA) || UNITY_EDITOR
        T o = JsonUtility.FromJson<T>(File.ReadAllText(filename));
#else
        T o = Newtonsoft.Json.JsonConvert.DeserializeObject<T>(System.Text.Encoding.UTF8.GetString(UnityEngine.Windows.File.ReadAllBytes(filename)));
#endif
        return o;
    }

    public void Load()
    {
        Debug.Log("MET archive loading from path: " + jsonPath);
        if (loadDxtObjects)
        {
            dxtObjects = LoadJson<DXTObjectList>(jsonPath + "/dxtObjects.json");
            dxtObjects.PrepareData();
            Debug.Log("MET DXT Objects loaded: " + dxtObjects.items.Length);
        }
        if (loadTags)
        {
            tags = LoadJson<TagList>(jsonPath + "/tagsIndexed.json");
            tags.PrepareData();
            Debug.Log("MET tags loaded: " + tags.items.Length);
        }
        dataLoaded = true;
    }

    // Singleton - rarely needed though
    private static MET _inst = null;
    private bool _loaded = false;
    public static MET Instance
    {
        get
        {
            if (!_inst)
                _inst = Resources.FindObjectsOfTypeAll<MET>().FirstOrDefault();
            if (!_inst.Loaded)
                _inst.Load();
            return _inst;
        }
    }
    public bool Loaded { get { return _loaded; } }
}
