﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

namespace eMPlus
{
    public static class ObjectCloner
    {
        public static bool debug = false;
        public static T GetCopyOf<T>(this Component comp, T other) where T : Component
        {
            BindingFlags pflags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Default | BindingFlags.DeclaredOnly;
            BindingFlags fflags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Default | BindingFlags.DeclaredOnly | BindingFlags.Instance;

            Type type = comp.GetType();
            if (type != other.GetType()) return null; // type mis-match
            PropertyInfo[] pinfos = type.GetProperties(pflags);
            foreach (var pinfo in pinfos)
            {
                if (pinfo.CanWrite)
                {
                    if (debug)
                        Debug.Log("property: " + pinfo);
                    try
                    {
                        pinfo.SetValue(comp, pinfo.GetValue(other, null), null);
                    }
                    catch { } // In case of NotImplementedException being thrown. For some reason specifying that exception didn't seem to catch it, so I didn't catch anything specific.
                }
            }
            FieldInfo[] finfos = type.GetFields(fflags);
            foreach (var finfo in finfos)
            {
                if (debug)
                    Debug.Log("field: " + finfo);
                finfo.SetValue(comp, finfo.GetValue(other));
            }
            return comp as T;
        }

        // typical usage with: Type[] skipList = { typeof(Transform), typeof(CaveCameraRig), typeof(AudioListener), typeof(FlareLayer), typeof(UnityEngine.GUILayer) };
        public static GameObject Clone(GameObject original, Type[] skipList)
        {
            GameObject clone = new GameObject();
            foreach (Component comp in original.GetComponents<Component>())
            {
                if (!Array.Exists(skipList, element => element == comp.GetType()))
                {
                    Component ccomp = clone.AddComponent(comp.GetType());
                    if (debug)
                        Debug.Log("cloning component of type: " + comp.GetType() + " to " + ccomp.GetType());
                    if (ccomp) ccomp.GetCopyOf(comp);
                } else
                {
                    if (debug)
                        Debug.Log("skipping component of type: " + comp.GetType());
                }
            }
            return clone;
        }
    }
}
