﻿using UnityEngine;
using UnityEngine.Networking;
using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;

public class SyncHub : MonoBehaviour {

    public class SyncMessageType
    {
        public static short Sync = 1000;
    }

    class SyncMessage : MessageBase
    {
        public float serverTime;
        public byte[] payload;
    }

    class SyncServer : NetworkServerSimple
    {
        public float serverTime = 0;
        protected BinaryFormatter formatter;

        public SyncServer() : base()
        {
            formatter = new BinaryFormatter();
        }
        public override void OnConnected(NetworkConnection conn)
        {
            Debug.Log("connected...");
            conn.InvokeHandlerNoData(MsgType.Connect);
        }
        public long Update(object payload)
        {
            serverTime = Time.unscaledTime;
            SyncMessage msg = new SyncMessage();
            msg.serverTime = serverTime;

            long payloadSize = 0;
            if (payload != null)
            {
                var stream = new System.IO.MemoryStream();
                formatter.Serialize(stream, payload);
                msg.payload = stream.GetBuffer();
                payloadSize = stream.Length;
            } else
            {
                msg.payload = null;
            }

            foreach (var conn in connections)
            {
                if (conn == null) continue;
                conn.Send(SyncMessageType.Sync, msg);
            }

            base.Update();

            return payloadSize;
        }
    }

    NetworkClient client = null;
    SyncServer server = null;

    public bool isServer = false;

    public string host = "192.168.1.71";
    public int port = 7889;
    public int connectionCount = 0;
    public bool isConnecting = false;
    public bool debug = false;

    public object payload = null;
    protected long payloadSize = 0;

    bool _isServer = false;

    public float serverTime;

    void Start() {
        var ips = GetLocalIPAddresses();
        if (Array.Exists(ips, element => element == host))
        {
            isServer = true;
        }
        _isServer = isServer;

        if (!_isServer)
        {
            StartClient();
        }
        else
        {
            host = "S:";
            foreach (var ip in ips) host = host + " " + ip;
            StartServer();
        }
    }

    void Update() {
        if (_isServer && server != null)
        {
            payloadSize = server.Update(payload);
            serverTime = server.serverTime;
        }
    }

    private void OnGUI()
    {
        if (_isServer && server != null)
            GUI.Label(new Rect(300, 100, 200, 100), "Server connections: " + server.connections.Count);
        if (_isServer || debug)
            GUI.Label(new Rect(100, 100, 200, 100), "host: " + host + "\nport: " + port + "\nconnections: " + connectionCount + "\n\nserverTime: " + serverTime + "\npayload: " + payloadSize + " bytes");
    }

    public void StartClient()
    {
        Debug.Log("Starting Client");
        if (client == null)
        {
            client = new NetworkClient();
            client.RegisterHandler(MsgType.Connect, OnConnectedToServer);
            client.RegisterHandler(MsgType.Disconnect, OnDisconnectedFromServer);
            client.RegisterHandler(SyncMessageType.Sync, OnSyncReceived);
        }
        client.Connect(host, port);
        isConnecting = true;
    }

    public void StartServer()
    {
        server = new SyncServer();
        server.RegisterHandler(MsgType.Connect, OnConnectedToClient);
        server.RegisterHandler(MsgType.Disconnect, OnDisconnectedFromClient);
        bool started = server.Listen(port);
        Debug.Log("Started Server: " + started);
    }

    public void OnSyncReceived(NetworkMessage netMsg)
    {
        SyncMessage msg = netMsg.ReadMessage<SyncMessage>();
        serverTime = msg.serverTime;
        BinaryFormatter formatter = new BinaryFormatter();
        var stream = new System.IO.MemoryStream(msg.payload);
        payload = formatter.Deserialize(stream);
//        Debug.Log("SYNC " + serverTime);
    }

    public void OnConnectedToServer(NetworkMessage netMsg)
    {
        Debug.Log("OnConnectedToServer");
        connectionCount++;
        isConnecting = false;
    }

    public void OnDisconnectedFromServer(NetworkMessage netMsg)
    {
        Debug.Log("OnDisconnectedFromServer");
        if (!isConnecting)
            connectionCount--;
        if (NetworkTransport.RemoveHost(0));
        StartClient();
    }

    public void OnConnectedToClient(NetworkMessage netMsg)
    {
        Debug.Log("OnConnectedToClient");
        connectionCount++;
    }

    public void OnDisconnectedFromClient(NetworkMessage netMsg)
    {
        Debug.Log("OnDisconnectedFromClient");
        connectionCount--;
    }

    public static string[] GetLocalIPAddresses()
    {
        List<string> ret = new List<string>();
        var nics = System.Net.NetworkInformation.NetworkInterface.GetAllNetworkInterfaces();
        foreach (var nic in nics)
        {
            if (nic.OperationalStatus != System.Net.NetworkInformation.OperationalStatus.Up) continue;
            var unicasts = nic.GetIPProperties().UnicastAddresses;
            foreach (System.Net.NetworkInformation.UnicastIPAddressInformation ip in unicasts)
            {
                if (ip.Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                {
                    Debug.Log(ip.Address.ToString() + " / " + nic.OperationalStatus.ToString());
                    ret.Add(ip.Address.ToString());
                }
            }
        }
        return ret.ToArray();
    }
}