﻿Shader "Hidden/ViosoDomeWarpBlendShader"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
        _LeftTex("Texture", 2D) = "red" {}
        _RightTex("Texture", 2D) = "green" {}
        _TopTex("Texture", 2D) = "black" {}
        _BottomTex("Texture", 2D) = "black" {}
        _WarpTex ("Warp Texture", 2D) = "white" {}
		_BlendTex ("Blend Texture", 2D) = "white" {}
        _Border ("Border Width", float) = 0.0
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
            sampler2D _LeftTex;
            sampler2D _RightTex;
            sampler2D _TopTex;
            sampler2D _BottomTex;
            sampler2D _WarpTex;
			sampler2D _BlendTex;
            float _Border;

            fixed4 fragDome(float2 iuv)
            {
                float2 uv = (iuv + float2(-0.5, -0.5)) * 2.0;
                uv = uv / (1.0 - _Border);
                float luv = length(uv);
                if (luv > 1.0) return fixed4(0,0,0,1);
                float2 angle = float2(atan2(uv.y, uv.x) + UNITY_PI * 0.5, (1.0 - luv) * UNITY_PI * 0.5);

                float x = sin(angle.x) * cos(angle.y);
                float z = -sin(angle.y);
                float y = -cos(angle.x) * cos(angle.y);
                float3 coord = float3(x, y, z);
                float a = sqrt(0.5);
                float3x3 rot = float3x3(-a, 0, -a, 0, 1, 0, -a, 0, a);
                float3 coordr = mul(coord, rot);
                x = coordr.x; y = coordr.y; z = coordr.z;

                float absX = abs(x);
                float absY = abs(y);
                float absZ = abs(z);
                float maxAxis, uc, vc;

                // POSITIVE X
                if (x > 0 && absX >= absY && absX >= absZ) {
                    maxAxis = absX;
                    uc = -z;
                    vc = y;
                    uv.x = 0.5f * (uc / maxAxis + 1.0f);
                    uv.y = 0.5f * (vc / maxAxis + 1.0f);
                    return tex2D(_LeftTex, uv);
                }
                // POSITIVE Y
                else if (y > 0 && absY >= absX && absY >= absZ) {
                    maxAxis = absY;
                    uc = x;
                    vc = -z;
                    uv.y = 1.0f - 0.5f * (uc / maxAxis + 1.0f);
                    uv.x = 0.5f * (vc / maxAxis + 1.0f);
                    return tex2D(_TopTex, uv);
                }
                // NEGATIVE Y
                else if (y <= 0 && absY >= absX && absY >= absZ) {
                    maxAxis = absY;
                    uc = x;
                    vc = z;
                    uv.y = 1.0 - 0.5f * (uc / maxAxis + 1.0f);
                    uv.x = 1.0 - 0.5f * (vc / maxAxis + 1.0f);
                    return tex2D(_BottomTex, uv.yx);
                }
                // NEGATIVE Z
                else if (z <= 0 && absZ >= absX && absZ >= absY) {
                    maxAxis = absZ;
                    uc = -x;
                    vc = y;
                    uv.x = 0.5f * (uc / maxAxis + 1.0f);
                    uv.y = 0.5f * (vc / maxAxis + 1.0f);
                    return tex2D(_RightTex, uv);
                }

                return fixed4(1,1,0,1);
            }

			fixed4 frag (v2f i) : SV_Target
			{
				float4 warpTex = tex2D(_WarpTex, float2(i.uv.x, 1.0 - i.uv.y));
                if (warpTex.z < 0.5) return fixed4(0,0,0,1);
				fixed4 blendTex = tex2D(_BlendTex, float2(i.uv.x, 1.0 - i.uv.y));

                //float2 uv = (float2(warpTex.x, 1.0 - warpTex.y) - 0.5) / (1.0 - _Border) + 0.5;
                float2 uv = float2(warpTex.x, 1.0 - warpTex.y);
                fixed4 col = fragDome(uv);
				col.rgb *= blendTex.rgb;
				return col;
			}
			ENDCG
		}
	}
}
