﻿Shader "Hidden/ViosoWarpBlendShader"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
		_WarpTex ("Warp Texture", 2D) = "white" {}
		_BlendTex ("Blend Texture", 2D) = "white" {}
        _Border("Border Width", float) = 0.0
    }
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			sampler2D _WarpTex;
			sampler2D _BlendTex;
            float _Border;

			fixed4 frag (v2f i) : SV_Target
			{
				float4 warpTex = tex2D(_WarpTex, float2(i.uv.x, 1.0 - i.uv.y));
                if (warpTex.z < 0.5) return fixed4(0,0,0,1);
				fixed4 blendTex = tex2D(_BlendTex, float2(i.uv.x, 1.0 - i.uv.y));

                float2 uv = (warpTex.xy + float2(-0.5, -0.5)) * 2.0;
                uv = uv / (1.0 - _Border);
                if (length(uv) > 1.0) return fixed4(0, 0, 0, 1);
                uv = uv * 0.5 + float2(0.5, 0.5);

				fixed4 col = tex2D(_MainTex, float2(uv.x, 1.0 - uv.y));
				col.rgb *= blendTex.rgb;
				return col;
			}
			ENDCG
		}
	}
}
