﻿using System;
using System.IO;
using UnityEngine;

namespace eMPlus.Vioso
{
    public class ViosoVwfWarpCamera : MonoBehaviour
    {
        public Material material;
        public RenderTexture renderTexture;

        public string fileName = "C:/ProgramData/AV Stumpfl/Wings 6/Vioso/_Startup.vwf";
        public bool smoothBorder = true;
        public float borderPercent = 2;
        public bool debug = true;

        ViosoVwfReader vwfReader;
        GameObject contentCam;

        void Start()
        {
            if (renderTexture == null)
                renderTexture = new RenderTexture(4096, 4096, 24);
            if (material == null)
                material = new Material(Shader.Find("Hidden/ViosoWarpBlendShader"));

            vwfReader = new ViosoVwfReader();
            if (!File.Exists(fileName)) fileName = "test.vwf";
            if (!File.Exists(fileName))
            {
                Debug.Log("no VWF file found, continuing without any warp/blend");
                return;
            }
            if (vwfReader.ReadVwf(fileName))
            {
                Type[] skipList = { typeof(Transform), typeof(eMPlus.Vioso.ViosoVwfWarpCamera) };
                contentCam = eMPlus.ObjectCloner.Clone(gameObject, skipList);
                contentCam.name = "Content Camera (cloned)";
                contentCam.transform.parent = transform;
                contentCam.transform.localPosition = Vector3.zero;
                contentCam.transform.localRotation = Quaternion.identity;

                Type[] keepList = { typeof(Transform), typeof(eMPlus.Vioso.ViosoVwfWarpCamera), typeof(Camera) };
                foreach (Component comp in gameObject.GetComponents<Component>())
                {
                    if (!Array.Exists(keepList, element => element == comp.GetType()))
                    {
                        Destroy(comp);
                    }
                }

                var ccam = contentCam.GetComponent<Camera>();
                var cam = GetComponent<Camera>();

                ccam.CopyFrom(cam);
                ccam.targetTexture = renderTexture;
                cam.targetTexture = null;
            }
        }

        void Update()
        {
            if (Input.GetKeyDown(KeyCode.Space)) debug = !debug;
        }

        public void OnGUI()
        {
            if (!debug)
                return;

            int sw = Screen.width / 2;
            int sh = Screen.height / 3;

            GUI.Label(new Rect(sw + 10, sh - 40, 400, 30), "File: " + fileName);
            vwfReader.ShowDebugGUI(sw, sh);
        }

        void OnRenderImage(RenderTexture src, RenderTexture dest)
        {
            if (vwfReader.blendTextures.Count > 0)
                material.SetTexture("_BlendTex", vwfReader.blendTextures[0]);
            if (vwfReader.warpTextures.Count > 0)
                material.SetTexture("_WarpTex", vwfReader.warpTextures[0]);
            material.SetFloat("_Border", smoothBorder ? 0.01f * borderPercent : 0f);
            Graphics.Blit(renderTexture, dest, material);
        }
    }
}
