﻿using UnityEngine;
using System.IO;
using eMPlus.Dome;

namespace eMPlus.Vioso
{
    public class ViosoVwfWarpDomeRig : MonoBehaviour
    {
        DomeCameraRig cameraRig;
        Material material;
        ViosoVwfReader vwfReader;

        public string fileName = "C:/ProgramData/AV Stumpfl/Wings 6/Vioso/_Startup.vwf";
        public bool smoothBorder = true;
        public float borderPercent = 2;
        public bool debug = false;
        bool rigInitialized = false;

        void Start()
        {
            vwfReader = new ViosoVwfReader();
            material = new Material(Shader.Find("Hidden/ViosoDomeWarpBlendShader"));
            if (!File.Exists(fileName)) fileName = "test.vwf";
            if (!File.Exists(fileName))
            {
                Debug.Log("no VWF file found, continuing without any warp/blend");
                return;
            }
            cameraRig = gameObject.AddComponent<DomeCameraRig>();
            cameraRig.debug = debug;
        }

        void Update()
        {
            if (Input.GetKeyDown(KeyCode.Space)) debug = !debug;
            if (Input.GetKeyDown(KeyCode.B)) smoothBorder = !smoothBorder;
            if (!cameraRig) return;

            if (cameraRig.isInitialized && !rigInitialized)
            {
                material.SetTexture("_LeftTex", cameraRig.renderTextures[0]);
                material.SetTexture("_RightTex", cameraRig.renderTextures[1]);
                material.SetTexture("_TopTex", cameraRig.renderTextures[2]);
                material.SetTexture("_BottomTex", cameraRig.renderTextures[3]);

                rigInitialized = true;
                if (vwfReader.ReadVwf(fileName))
                {
                    GetComponent<Camera>().enabled = true;
                }
            }
        }

        public void OnGUI()
        {
            if (!debug)
                return;

            int sw = Screen.width / 2;
            int sh = Screen.height / 3;

            GUI.Label(new Rect(sw + 10, sh - 70, 400, 30), "CameraRig: " + (cameraRig != null && cameraRig.isInitialized ? "initialized" : (cameraRig != null ? "N/A" : "null")));
            GUI.Label(new Rect(sw + 10, sh - 40, 400, 30), "VWF File: " + fileName);
            vwfReader.ShowDebugGUI(sw, sh);
        }

        void OnRenderImage(RenderTexture src, RenderTexture dest)
        {
            if (!cameraRig)
            {
                Graphics.Blit(src, dest);
                return;
            };
            if (vwfReader.blendTextures.Count > 0)
                material.SetTexture("_BlendTex", vwfReader.blendTextures[0]);
            if (vwfReader.warpTextures.Count > 0)
                material.SetTexture("_WarpTex", vwfReader.warpTextures[0]);
            material.SetFloat("_Border", smoothBorder ? 0.01f * borderPercent : 0f);

            Graphics.Blit(src, dest, material);
        }
    }
}
